SystemOrganization addCategory: #'AJP-Pharo-Core'!

AJPAdaptor subclass: #AJPPharoAdaptor
	instanceVariableNames: 'server wasRunning'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

!AJPPharoAdaptor methodsFor: 'private-controlling' stamp: 'pmm 10/28/2010 07:11'!
basicStart
	wasRunning := true.
	server := AJPPharoServer new.
	server startOn: self port using: self.! !

!AJPPharoAdaptor methodsFor: 'private-controlling' stamp: 'pmm 10/28/2010 07:11'!
basicStop
	wasRunning := false.
	server stop.
	server := nil! !

!AJPPharoAdaptor methodsFor: 'initialization' stamp: 'pmm 10/28/2010 07:13'!
initialize
	super initialize.
	wasRunning := false! !

!AJPPharoAdaptor methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:51'!
isRunning
	^server isNil not
		and: [ server isStarted ]! !

!AJPPharoAdaptor methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:51'!
isStarting
	^server isNil not
		and: [ server isStarting ]! !

!AJPPharoAdaptor methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:52'!
isStopped
	^server isNil
		or: [ server isStopped ]! !

!AJPPharoAdaptor methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:52'!
isStopping
	^server isNil not
		and: [ server isStopping ]! !

!AJPPharoAdaptor methodsFor: 'private' stamp: 'pmm 11/7/2010 10:44'!
shutDown
	| oldValue |
	oldValue := wasRunning.
	self stop.
	wasRunning := oldValue! !

!AJPPharoAdaptor methodsFor: 'private' stamp: 'pmm 10/28/2010 07:11'!
startUp
	wasRunning ifTrue: [
		self start ]! !

!YBuffer methodsFor: '*ajp-pharo-core' stamp: 'pmm 2/11/2012 21:49'!
sendToSocket: aSocket
	self subclassResponsibility! !

!YFullBuffer methodsFor: '*ajp-pharo-core' stamp: 'pmm 2/11/2012 22:06'!
sendToSocket: aSocket
	| startIndex toWrite |
	startIndex := position.
	toWrite := limit - position.
	[ toWrite > 0 ] whileTrue: [
		| written |
		written := aSocket sendSomeData: buffer startIndex: startIndex count: toWrite.
		startIndex := startIndex + written.
		toWrite := toWrite - written  ]! !

!YSubBuffer methodsFor: '*ajp-pharo-core' stamp: 'pmm 2/11/2012 22:06'!
sendToSocket: aSocket
	|  startIndex toWrite |
	startIndex := start + position - 1.
	toWrite := limit - 1.
	[ toWrite > 0 ] whileTrue: [
		| written |
		written := aSocket sendSomeData: buffer startIndex: startIndex count: toWrite.
		startIndex := startIndex + written.
		toWrite := toWrite - written  ]! !

AJPServer subclass: #AJPPharoServer
	instanceVariableNames: 'shutdown'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

!AJPPharoServer class methodsFor: 'accessing' stamp: 'pmm 1/6/2009 07:03'!
bufferSize
	"the buffer size to be used on sockets"
	^16r2000! !

!AJPPharoServer methodsFor: 'private' stamp: 'pmm 10/31/2010 14:17'!
acceptLoopOn: aSocket
	[ [ self shouldStop not and: [ aSocket isValid ] ] whileTrue: [
		| connection |
		connection := aSocket waitForAcceptFor: 3.
		connection isNil ifFalse: [
			self startWorkerOn: (self configureSocket: connection) ]
	 ] ] ensure: [
		aSocket destroy.
		shutdown signal ]! !

!AJPPharoServer methodsFor: 'private' stamp: 'pmm 2/9/2012 19:52'!
configureSocket: aSocket
	"http://baus.net/on-tcp_cork"
	^ aSocket
		setOption: 'TCP_NODELAY' value: 1;
		"setOption: 'TCP_CORK' value: 1;"
		"setOption: 'TCP_NOWAIT' value: 1;"
		"setOption: 'TCP_NOPUSH' value: 1;"
		"setOption: 'TCP_MAXSEG' value: AJPPharoServer bufferSize + (320 / 8);"
		setOption: 'SO_SNDBUF' value: AJPPharoServer bufferSize;
		setOption: 'SO_RCVBUF' value: AJPPharoServer bufferSize;
		yourself! !

!AJPPharoServer methodsFor: 'private' stamp: 'pmm 10/31/2010 14:16'!
newTcpSocket
	^ self configureSocket: Socket newTCP! !

!AJPPharoServer methodsFor: 'actions' stamp: 'pmm 10/30/2010 22:54'!
removeWorker: aLoop
	mutex critical: [
		workers remove: aLoop ifAbsent: [ "ignore" ] ] ! !

!AJPPharoServer methodsFor: 'actions' stamp: 'pmm 10/8/2010 11:34'!
startOn: aPort using: anAdapter
	| socket process |
	self beStarting.
	shutdown := Semaphore new.
	adapter := anAdapter.
	socket := self newTcpSocket.
	socket listenOn: aPort backlogSize: self backlogSize.
	process := [ self acceptLoopOn: socket ] newProcess.
	process priority: Processor highIOPriority.
	process name: 'AJP server'.
	process resume.
	self beStarted! !

!AJPPharoServer methodsFor: 'private' stamp: 'pmm 10/7/2010 13:39'!
startWorkerOn: aSocket
	| worker process |
	mutex critical: [
		shouldStop ifTrue: [
			aSocket closeAndDestroy: 30.
			^self ].
		worker := AJPPharoSocketLoop on: aSocket server: self adapter: adapter.
		process := [ worker loop ] newProcess.
		process priority: Processor highIOPriority.
		process name: 'AJP worker ', (workers size  + 1) printString.
		workers add: worker ].
	process resume! !

!AJPPharoServer methodsFor: 'actions' stamp: 'pmm 10/8/2010 11:36'!
stop
	super stop.
	shutdown wait! !

AJPDefaultDataStream subclass: #AJPPharoBufferDataStream
	instanceVariableNames: 'socket readStream writeStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

!AJPPharoBufferDataStream class methodsFor: 'instance creation' stamp: 'pmm 10/30/2010 18:52'!
on: aSocket
	^self basicNew initializeOn: aSocket! !

!AJPPharoBufferDataStream methodsFor: 'public' stamp: 'pmm 10/30/2010 18:46'!
destroy
	socket ifNotNilDo: [ :s |
		"#destroy may have been called several times concurrently"
		s closeAndDestroy: 30. ].
	socket := nil.
	readStream := nil.
	writeStream := nil
	! !

!AJPPharoBufferDataStream methodsFor: 'public' stamp: 'pmm 2/11/2012 22:00'!
disableDelays
	writeStream disableDelays! !

!AJPPharoBufferDataStream methodsFor: 'public' stamp: 'pmm 2/11/2012 22:00'!
enableDelays
	writeStream enableDelays! !

!AJPPharoBufferDataStream methodsFor: 'public' stamp: 'pmm 2/11/2012 22:00'!
endOfPacket
	writeStream endOfPacket! !

!AJPPharoBufferDataStream methodsFor: 'public' stamp: 'pmm 10/30/2010 19:53'!
flush
	writeStream flush! !

!AJPPharoBufferDataStream methodsFor: 'hinting' stamp: 'pmm 10/30/2010 18:32'!
hintNewBinaryPacket
	readStream readPacket! !

!AJPPharoBufferDataStream methodsFor: 'hinting' stamp: 'pmm 10/30/2010 18:32'!
hintNewTextPacket
	readStream readPacket! !

!AJPPharoBufferDataStream methodsFor: 'initialize-release' stamp: 'pmm 10/30/2010 19:08'!
initializeOn: aSocket
	self initialize.
	socket := aSocket.
	readStream := AJPPharoSocketReadStream on: aSocket.
	writeStream := AJPPharoSocketWriteStream on: aSocket! !

!AJPPharoBufferDataStream methodsFor: 'testing' stamp: 'pmm 10/30/2010 19:06'!
isConnected
	"The stream is connected if the socket is."

	^socket notNil and: [ socket isConnected ]! !

!AJPPharoBufferDataStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 18:32'!
readByte
	^ readStream nextByte! !

!AJPPharoBufferDataStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 22:03'!
readChunk: anInteger into: aCollection startingAt: startIndex
	readStream readChunk: anInteger into: aCollection startingAt: startIndex! !

!AJPPharoBufferDataStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 18:32'!
readStringWithSizeHighByte: aByte
	"reads a String with the high byte of the size already read
	otherwise the same as #readString"
	| result size |
	size := (aByte << 8) bitOr: (self readByte).
	size = 16rFFFF ifTrue: [
		^String new ].
	
	"convert bytes to Characters, we don't want to switch to text mode because that would reset the buffers"
	result := readStream nextCharacters: size.
	
	"skip 0 byte"
	readStream nextByte = 0
		ifFalse: [ self protocolError: '0x00 byte string terminator expected' ].
	^result! !

!AJPPharoBufferDataStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 18:32'!
writeByte: aByte
	writeStream writeByte: aByte! !

!AJPPharoBufferDataStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 18:32'!
writeBytes: aByteArray
	writeStream writeBytes: aByteArray! !

!AJPPharoBufferDataStream methodsFor: 'writing' stamp: 'pmm 12/26/2010 10:09'!
writeChunk: aBuffer
	writeStream writeChunk: aBuffer! !

!AJPPharoBufferDataStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 18:32'!
writeString: aString
	self writeInteger: aString size.
	writeStream writeString:  aString.
	self writeByte: 0! !

GRObject subclass: #AJPPharoDirectSocketStream
	instanceVariableNames: 'socket isMacOS'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

!AJPPharoDirectSocketStream class methodsFor: 'instance creation' stamp: 'pmm 2/11/2012 22:13'!
on: aSocket
	^self basicNew initializeOn: aSocket! !

!AJPPharoDirectSocketStream methodsFor: 'public' stamp: 'pmm 2/11/2012 21:44'!
disableDelays
	socket setOption: 'TCP_NODELAY' value: 1.
	isMacOS
		ifTrue: [
			socket setOption: 'TCP_NOWAIT' value: 1.
			socket setOption: 'TCP_NOPUSH' value: 0 ]
		ifFalse: [ socket setOption: 'TCP_CORK' value: 0 ]! !

!AJPPharoDirectSocketStream methodsFor: 'public' stamp: 'pmm 2/11/2012 21:43'!
enableDelays
	socket setOption: 'TCP_NODELAY' value: 0.
	isMacOS
		ifTrue: [
			socket setOption: 'TCP_NOWAIT' value: 0.
			socket setOption: 'TCP_NOPUSH' value: 1 ]
		ifFalse: [ socket setOption: 'TCP_CORK' value: 1 ]! !

!AJPPharoDirectSocketStream methodsFor: 'public' stamp: 'pmm 2/11/2012 22:34'!
endOfPacket
	"ignore"! !

!AJPPharoDirectSocketStream methodsFor: 'public' stamp: 'pmm 2/11/2012 22:32'!
flush
	"ignore"! !

!AJPPharoDirectSocketStream methodsFor: 'initialize-release' stamp: 'pmm 2/11/2012 21:40'!
initializeOn: aSocket
	self initialize.
	socket := aSocket.
	isMacOS := SmalltalkImage current os isMacOS.! !

!AJPPharoDirectSocketStream methodsFor: 'writing' stamp: 'pmm 2/11/2012 21:46'!
writeByte: aByte
	socket sendData: (ByteArray with: aByte)! !

!AJPPharoDirectSocketStream methodsFor: 'writing' stamp: 'pmm 2/11/2012 21:47'!
writeBytes: aByteArray
	socket sendData: aByteArray! !

!AJPPharoDirectSocketStream methodsFor: 'writing' stamp: 'pmm 2/11/2012 21:49'!
writeChunk: aBuffer
	aBuffer sendToSocket: socket! !

!AJPPharoDirectSocketStream methodsFor: 'writing' stamp: 'pmm 2/11/2012 21:47'!
writeString: aString
	socket sendData: aString! !

GRObject subclass: #AJPPharoSocketStream
	instanceVariableNames: 'socket buffer position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

AJPPharoSocketStream subclass: #AJPPharoSocketReadStream
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

!AJPPharoSocketReadStream methodsFor: 'private' stamp: 'pmm 10/30/2010 18:18'!
capacity
	"4 - 1 + 1"
	^limit - position + 1! !

!AJPPharoSocketReadStream methodsFor: 'private' stamp: 'pmm 10/30/2010 19:18'!
ensureCapacity: anInteger
	| capacity |
	capacity := self capacity.
	capacity < anInteger ifTrue: [
		anInteger > AJPPharoServer bufferSize ifTrue: [
			self socketError: 'data size ', anInteger printString, ' not supported' ].
		(position + anInteger - 1) <= AJPPharoServer bufferSize
			ifTrue: [
				self readntoBuffer: anInteger - capacity ]
			ifFalse: [
				"uncool"
				buffer copyFrom: position to: limit into: buffer startingAt: 1.
				limit := limit - position - 1.
				position := 1 ] ]! !

!AJPPharoSocketReadStream methodsFor: 'initialize-release' stamp: 'pmm 10/30/2010 19:44'!
initialize
	super initialize.
	limit := 0! !

!AJPPharoSocketReadStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 18:18'!
nextByte
	| data |
	self ensureCapacity: 1.
	data := buffer at: position.
	position := position + 1.
	^ data! !

!AJPPharoSocketReadStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 18:18'!
nextBytes: anInteger
	| data |
	self ensureCapacity: anInteger.
	data := buffer copyFrom: position to: position + anInteger.
	position := position + anInteger.
	^ data! !

!AJPPharoSocketReadStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 19:24'!
nextCharacters: anInteger
	| data |
	self ensureCapacity: anInteger.
	data := String new: anInteger.
	data replaceFrom: 1 to: anInteger with: buffer startingAt: position.
	position := position + anInteger.
	^ data! !

!AJPPharoSocketReadStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 22:05'!
readChunk: anInteger into: aCollection startingAt: startIndex
	self ensureCapacity: anInteger.
	aCollection replaceFrom: startIndex to: startIndex + anInteger - 1 with: buffer startingAt: position.
	position := position + anInteger! !

!AJPPharoSocketReadStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 22:36'!
readPacket
	position <= limit
		ifTrue: [
			"still data available, copy to start of buffer for further reads"
			buffer := buffer replaceFrom: 1 to: limit - position + 1 with: buffer startingAt: position.
			limit := limit - position + 1.
			position := 1 ]
		ifFalse: [
			| read |
			read := socket receiveAvailableDataInto: buffer startingAt: 1.
			[ read = 0 ] whileTrue: [
				socket waitForData.
				read := socket receiveAvailableDataInto: buffer startingAt: 1 ].
			position := 1.
			limit := read ]! !

!AJPPharoSocketReadStream methodsFor: 'private' stamp: 'pmm 10/30/2010 19:24'!
readntoBuffer: anInteger
	| read |
	read := socket  receiveAvailableDataInto: buffer startingAt: limit + 1.
	limit := limit + read.
	read < anInteger
		ifTrue: [
			socket waitForData.
			self readntoBuffer: anInteger - read ]! !

!AJPPharoSocketStream class methodsFor: 'instance creation' stamp: 'pmm 10/30/2010 18:53'!
on: aSocket
	^self basicNew initializeOn: aSocket! !

!AJPPharoSocketStream methodsFor: 'private' stamp: 'pmm 10/30/2010 18:18'!
ensureCapacity: anInteger
	self subclassResponsibility! !

!AJPPharoSocketStream methodsFor: 'initialize-release' stamp: 'pmm 10/30/2010 18:26'!
initializeOn: aSocket
	self initialize.
	socket := aSocket.
	position := 1.
	buffer := self newBuffer! !

!AJPPharoSocketStream methodsFor: 'private' stamp: 'pmm 10/30/2010 18:25'!
newBuffer
	^ByteArray new: AJPPharoServer bufferSize withAll: 0! !

!AJPPharoSocketStream methodsFor: 'private' stamp: 'pmm 4/23/2011 15:47'!
socketError: aString
	AJPPharoSocketError signal: aString! !

AJPPharoSocketStream subclass: #AJPPharoSocketWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

!AJPPharoSocketWriteStream methodsFor: 'public' stamp: 'pmm 2/11/2012 22:00'!
disableDelays
	"nothing just a hint"! !

!AJPPharoSocketWriteStream methodsFor: 'public' stamp: 'pmm 2/11/2012 22:00'!
enableDelays
	"nothing just a hint"! !

!AJPPharoSocketWriteStream methodsFor: 'public' stamp: 'pmm 2/11/2012 21:59'!
endOfPacket
	self sendBuffer! !

!AJPPharoSocketWriteStream methodsFor: 'private' stamp: 'pmm 10/30/2010 18:23'!
ensureCapacity: anInteger
	position + anInteger > AJPPharoServer bufferSize ifTrue: [
		self sendBuffer ]! !

!AJPPharoSocketWriteStream methodsFor: 'public' stamp: 'pmm 10/30/2010 18:19'!
flush
	self sendBuffer! !

!AJPPharoSocketWriteStream methodsFor: 'private' stamp: 'pmm 10/30/2010 22:27'!
sendBuffer
	socket sendData: buffer count: position - 1.
	position := 1! !

!AJPPharoSocketWriteStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 18:19'!
writeByte: aByte
	self ensureCapacity: 1.
	buffer at: position put: aByte.
	position := position + 1.! !

!AJPPharoSocketWriteStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 19:27'!
writeBytes: aByteArray
	self ensureCapacity: aByteArray size.
	buffer replaceFrom: position to: position + aByteArray size - 1 with: aByteArray startingAt: 1.
	position := position + aByteArray size! !

!AJPPharoSocketWriteStream methodsFor: 'writing' stamp: 'pmm 12/26/2010 13:09'!
writeChunk: aBuffer
	| size |
	size := aBuffer size.
	self ensureCapacity: size.
	aBuffer transferTo: buffer startingAt: position.
	position := position + size! !

!AJPPharoSocketWriteStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 19:27'!
writeString: aString
	self ensureCapacity: aString size.
	buffer replaceFrom: position to: position + aString size - 1 with: aString startingAt: 1.
	position := position + aString size.! !

AJPMimeParser subclass: #AJPPharoMimeParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

GRError subclass: #AJPPharoSocketError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

AJPAdapterLoop subclass: #AJPPharoSocketLoop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Pharo-Core'!

!AJPPharoSocketLoop methodsFor: 'private' stamp: 'pmm 10/31/2010 13:41'!
createBufferDataStreamOn: aSocket
	^ AJPPharoBufferDataStream on: aSocket! !

!AJPPharoSocketLoop methodsFor: 'initialize-release' stamp: 'pmm 10/31/2010 13:44'!
initializeOn: aSocket server: aServer
	super
		initializeOn: (self createBufferDataStreamOn: aSocket)
		server: aServer! !

!AJPPharoSocketLoop methodsFor: 'private' stamp: 'pmm 8/30/2009 12:50'!
readIgnoringTimeout
	[ 
		| next |
		(shouldStop or: [ (stream isConnected) not ])
			ifTrue: [ AJPLoopTerminatedError signal ].
		next := [ stream readByte ]
			on: ConnectionTimedOut
			do: [ :error | nil ].
		next isNil ifFalse: [ ^next ] ] repeat! !
